@@ -20,6 +20,17 @@ class BaseClient:
     def __init__(self, config: Config):
         self.client = None
         self.config = config
+        self.stream = False
+
+
+class BaseInvoke:
+    def __init__(self, config: Config, method):
+        self.config = config
+        self._method = method
+        self._client_provider = None
+        self._client_title = None
+        self._client_version = None
+        self._uses_protobuf = False
 
     def dict_to_json(self, dict_):
         result = {}
@@ -36,21 +47,46 @@ class BaseClient:
 
         return result
 
+    def _format_kwargs(self, kwargs):
+        if self._uses_protobuf:
+            formatted_kwargs = json.loads(
+                json_format.MessageToJson(kwargs["request"].__dict__["_pb"])
+            )
+        else:
+            formatted_kwargs = self.dict_to_json(kwargs)
+
+        return formatted_kwargs
+
     def _format_payload(
-        self, client_title, client_version, start_time, end_time, query, response
+        self,
+        client_provider,
+        client_title,
+        client_version,
+        start_time,
+        end_time,
+        query,
+        response,
     ):
         response_json = self.response_to_json(response)
 
         payload = {
             "attribution": self.config.attribution,
             "conversation": {
-                "client": {"title": client_title, "version": client_version},
+                "client": {
+                    "provider": client_provider,
+                    "title": client_title,
+                    "version": client_version,
+                },
                 "query": query,
                 "response": response_json,
             },
             "meta": {
                 "api": {"key": self.config.api_key},
-                "sdk": {"version": self.config.version},
+                "fnfg": {
+                    "exc": None,
+                    "status": "succeeded",
+                },
+                "sdk": {"client": "python", "version": self.config.version},
             },
             "time": {"end": end_time, "start": start_time},
             "tx": {"uuid": str(self.config.tx_uuid)},
@@ -58,37 +94,14 @@ class BaseClient:
 
         return payload
 
-    def _invoke(
-        self, client_title, client_version, method, kwargs, uses_protobuf=False
-    ):
-        start = time.time()
-
-        formatted_kwargs = dict(kwargs)
-        if uses_protobuf:
-            formatted_kwargs = json.loads(
-                json_format.MessageToJson(formatted_kwargs["request"].__dict__["_pb"])
-            )
-
-        raw_response = method(**kwargs)
-
+    def _format_response(self, raw_response):
         formatted_response = copy.deepcopy(raw_response)
-        if uses_protobuf:
+        if self._uses_protobuf:
             formatted_response = json.loads(
                 json_format.MessageToJson(formatted_response.__dict__["_pb"])
             )
 
-        Collector(self.config).fire_and_forget(
-            self._format_payload(
-                client_title,
-                client_version,
-                start,
-                time.time(),
-                formatted_kwargs,
-                formatted_response,
-            )
-        )
-
-        return raw_response
+        return formatted_response
 
     def list_to_json(self, list_):
         result = []
@@ -125,6 +138,16 @@ class BaseClient:
 
         return result
 
+    def set_client(self, provider, title, version):
+        self._client_provider = provider
+        self._client_title = title
+        self._client_version = version
+        return self
+
+    def uses_protobuf(self):
+        self._uses_protobuf = True
+        return self
+
 
 class BaseProvider:
     def __init__(self, parent):
