@@ -11,6 +11,7 @@ class Config:
     def __init__(self):
         self.api_key = None
         self.attribution = None
-        self.secs_requests_timeout = 5
+        self.secs_post_timeout = 5
+        self.raise_final_request_attempt = True
         self.tx_uuid = None
-        self.version = "0.2.3"
+        self.version = "0.3.5"
